<?php
// $Id$
/**
 * @package	SlUser
 * @subpackage SlRegUser
 * @copyright	Copyright (C) 2009 Wene / ssm2017 Binder (S.Massiaux). All rights reserved.
 * @license		GNU/GPL, http://www.gnu.org/licenses/gpl-2.0.html
 * SlUser is free software. This version may have been modified pursuant to the GNU General Public License,
 * and as distributed it includes or is derivative of works licensed under the GNU General Public License
 * or other free or open source software licenses.
 */

/**
* Route commands
*/
function slreguser_inworld_controller($cmd, &$sl, $args) {
  $response = array();
  switch ($cmd) {
    case 'checkRegistration':
      $response = slreguser_check_registration($sl, $args);
      break;

    case 'registerUser':
      $response = slreguser_register_user($sl, $args);
      break;

    default:
      $response['status'] = FALSE;
      $response['message'] = t('Oops! Unknown command:'). $cmd;
      break;
  }
  $sl->response['status'] = $response['status'];
  $sl->response['message'] = $response['message'];
}

/**
* Check user registration
*/
function slreguser_check_registration(&$sl, $args) {
  $response = array();

  // check the terminal password
  if (!module_invoke('sluser', 'terminal_check_password', $args['password'], $args['keypass'])) {
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('Wrong terminal password.');
    return $response;
  }

  // check if slreguser parameters allow registration
  if (variable_get('slreguser_allow_registration', 1) == 0) {
    $response['status'] = FALSE;
    $response['message'] = 'registration closed;'. t('Registration is closed.');
    return $response;
  }

  // check if main site parameters allow registration
  if (variable_get('user_register', 0) == 0) {
    $response['status'] = FALSE;
    $response['message'] = 'registration closed;'. t('Registration is closed.');
    return $response;
  }

  // check if the user already have a sluser account
  $values = array(
    'uuid' =>$args['user_key']
  );
  $sluser = sluser_load($values);
  if ($sluser->id) {
    $account = user_load($sluser->uid);
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('You already have an account named "%name" on the site.', array('%name' => $account->name));
    return $response;
  }

  // check if the user name is already registered on the site
  $account = user_load(array('name' =>$args['website_username']));
  if ($account->uid != 0) {
    $response['status'] = FALSE;
    $response['message'] = 'already registered;'. t('User with this name is already registered.');
    return $response;
  }

  // user is not registered so return success
  $response['status'] = TRUE;
  $response['message'] = 'success;'. t('Start registering process.');
  return $response;
}

/**
* Register the user
*/
function slreguser_register_user(&$sl, $args) {
  $reponse = array();

  // check the terminal password
  if (!module_invoke('sluser', 'terminal_check_password', $args['password'], $args['keypass'])) {
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('Wrong terminal password.');
    return $response;
  }

  // check if the user already have a sluser account
  $values = array(
    'uuid' =>$args['user_key']
  );
  $sluser = sluser_load($values);
  if ($sluser->id) {
    $account = user_load($sluser->uid);
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('You already have an account named "%name" on the site.', array('%name' => $account->name));
    return $response;
  }

  // Verify the syntax of the given name.
  $check_username = user_validate_name($args['website_username']);
  if ($check_username != "") {
    $response['status'] = FALSE;
    $response['message'] = "error;". $check_username;
    return $response;
  }

  // verify the email
  $check_email = user_validate_mail($args['email']);
  if ($check_email != "") {
    $response['status'] = FALSE;
    $response['message'] = "error;". $check_email;
    return $response;
  }

  $name  = $args['website_username'];
  $mail    = $args['email'];
  $pass   = $args['pass'];
  $status = 1;

  // define if the user needs to validate email
  if (variable_get('user_register', 0) == 2 || variable_get('user_email_verification', TRUE)) {
    $status = 0;
  }

  // register the user
  $values = array(
    'name'  =>$name,
    'mail'    =>$mail,
    'pass'   =>$pass,
    'status' =>$status,
    'roles'   =>array_filter(variable_get('sluser_default_roles', array()))
  );
  $account = user_save('', $values);
  if (!$account) {
    $response['status'] = FALSE;
    $response['message'] = "error;". t("Error saving user account.");
    return $response;
  }

  // add or update the sluser
  $args['uid'] = $account->uid;
  $args['name'] = $args['sl_username'];
  $args['uuid'] = $args['user_key'];
  unset($args['sl_username']);
  $saved = sluser_save($args);
  $response['status'] = $saved['status'];
  $response['message'] = $saved['message'];
  if ($saved['status'] == FALSE) {
    return $response;
  }

  watchdog('user', 'SlRegUser inworld : New user: %name (%email).', array('%name' => $name, '%email' => $mail), WATCHDOG_NOTICE, l(t('edit'), 'user/'. $account->uid .'/edit'));

  // Add plain text password into user account to generate mail tokens.
    $account->password = $pass;

  // user is registered so check if the user needs a confirmation email
  if (variable_get('user_email_verification', TRUE) && variable_get('user_register', 0) < 2) {
    // Create new user account, no administrator approval required but need to confirm email.
    _user_mail_notify('register_no_approval_required', $account);
    $response['status'] = TRUE;
    $response['message'] = "success need activate;". t('Your password and further instructions have been sent to your e-mail address.');
    return $response;
  }
  else if (variable_get('user_register', 0) == 2) {
    // Create new user account, administrator approval required.
    _user_mail_notify('register_pending_approval', $account);
    $response['status'] = TRUE;
    $response['message'] = "success need activate;". t('Thank you for applying for an account. Your account is currently pending approval by the site administrator.<br />In the meantime, a welcome message with further instructions has been sent to your e-mail address.');
    return $response;
  }
  else if ($status == 1) {
    // Create new user account, no administrator approval required.
    _user_mail_notify('register_no_approval_required', $account);
    $response['status'] = TRUE;
    $response['message'] = "success reg complete;". t('Your password and further instructions have been sent to your e-mail address.');
    return $response;
  }
}