<?php
// $Id: sluser.inworld.inc,v 1.7 2009/03/02 02:37:04 ssm2017binder Exp $
/**
 * @package	SlUser
 * @copyright	Copyright (C) 2009 Wene / ssm2017 Binder (S.Massiaux). All rights reserved.
 * @license		GNU/GPL, http://www.gnu.org/licenses/gpl-2.0.html
 * SlUser is free software. This version may have been modified pursuant to the GNU General Public License,
 * and as distributed it includes or is derivative of works licensed under the GNU General Public License
 * or other free or open source software licenses.
 */

/**
* Route commands
*/
function sluser_inworld_controller($cmd, &$sl, $args) {
  $response = array();
  switch ($cmd) {
    case 'registerTerminal':
    case 'updateTerminal':
      $response = sluser_register_terminal($sl, $args);
      break;

    case 'checkRegKey':
      $response = sluser_check_reg_key($sl, $args);
      break;

    default:
      $response['status'] = FALSE;
      $response['message'] = t('Oops! Unknown command:'). $cmd;
      break;
  }
  $sl->response['status'] = $response['status'];
  $sl->response['message'] = $response['message'];
}

/**
* Register the terminal
*/
function sluser_register_terminal(&$sl, $args) {
  $response = array();

  // check the terminal password
  if (!sluser_terminal_check_password($args['password'], $args['keypass'])) {
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('Wrong terminal password.');
    return $response;
  }

  $values = array(
    'name'      =>$sl->objectname,
    'uuid'        =>$sl->objectkey,
    'region'     =>$sl->region_name,
    'position'  =>strtr($sl->position, array('(' =>'', ')' =>'', ',' =>'/', ' '=>'')),
    'rpc_channel' =>$args['rpc_channel']
  );
  $registered = sluser_terminal_save($values);
  $response['status'] = $registered['status'];
  $response['message'] = $registered['message'];
  return $response;
}

/**
* Check the user's reg key
*/
function sluser_check_reg_key($sl, $args) {
  $response = array();

  // check the terminal password
  if (!sluser_terminal_check_password($args['password'], $args['keypass'])) {
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('Wrong terminal password.');
    return $response;
  }

  $user_key = $args['user_key'];
  $user_name = $args['user_name'];

  // check if the user is already registered
  $values = array(
    'name' =>$user_name
  );
  $sluser = sluser_load($values);
  if ($sluser->uid) {
    $account = user_load($sluser->uid);
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('You already have an account named !name registered with this avatar.', array('!name' =>$account->name)).';'.$user_key;
    return $response;
  }

  // get the user
  $values = array(
    'reg_key' =>$args['reg_key']
  );
  $sluser = sluser_load($values);
  if (!$sluser->id) {
    $response['status'] = FALSE;
    $response['message'] = 'error;'. t('Could not find user.').';'.$user_key;
    return $response;
  }

  // add values to the user
  $values = array(
    'uid' => $sluser->uid,
    'name' => $user_name,
    'uuid' => $user_key,
    'reg_key' =>$args['reg_key']
  );
  $registered = sluser_save($values);

  // change the role for the user
  $values = array(
    'uid'     =>$sluser->uid,
    'roles'   =>array_filter(variable_get('sluser_default_roles', array()))
  );
  $account = new stdclass;
  $account->uid = $sluser->uid;
  $account = user_save($account, $values);
  if (!$account) {
    $response['status'] = FALSE;
    $response['message'] = "error;". t("Error saving user account.").';'. $user_key;
    return $response;
  }
  
  $response['status'] = $registered['status'];
  $response['message'] = $registered['message'].';'. $user_key;
  return $response;
}