<?php
// $Id: sluser.users.inc,v 1.8 2009/03/02 02:37:04 ssm2017binder Exp $
/**
 * @package	SlUser
 * @copyright	Copyright (C) 2009 Wene / ssm2017 Binder (S.Massiaux). All rights reserved.
 * @license		GNU/GPL, http://www.gnu.org/licenses/gpl-2.0.html
 * SlUser is free software. This version may have been modified pursuant to the GNU General Public License,
 * and as distributed it includes or is derivative of works licensed under the GNU General Public License
 * or other free or open source software licenses.
 */

/**
* Implementation of hook_user().
*/
function sluser_user($op, &$edit, &$account, $category = NULL) {
  $admin = user_access('administer users');

  switch ($op) {

    case 'delete':
      // get the sluser
      $sluser = sluser_load(array('uid' =>$account->uid));
      if (!empty($sluser->id)) {
        sluser_delete($sluser->id);
        drupal_set_message(t('SlUser deleted.'));
      }
      break;

    case 'form':
      return sluser_edit_user_form($account, $edit);
      break;

    case 'insert':
      // generate a reg key
      sluser_generate_regkey($account);
      break;

    case 'view':
      sluser_view_user_form($account);
      break;

    case 'validate':
      return array(
        'sluser_allow_profile_view' => isset($edit['sluser_allow_profile_view']) ? $edit['sluser_allow_profile_view'] : variable_get('sluser_default_profile_view', 1),
        'sluser_allow_tab_view' => isset($edit['sluser_allow_tab_view']) ? $edit['sluser_allow_tab_view'] : variable_get('sluser_allow_tab_view', 1),
      );
      break;

    case 'insert':
      return array(
        $edit['sluser_allow_profile_view'] = 1,
        $edit['sluser_allow_tab_view'] = 1,
      );
    break;
  }
}

/**
* View user form
*/
function sluser_view_user_form(&$account) {
  $sluser = sluser_load(array('uid' =>$account->uid));
  if (user_edit_access($account) || $account->sluser_allow_profile_view) {
    // display the label
    if (($sluser->id && $sluser->name != '' && $sluser->uuid != '') || (user_edit_access($account))) {
      $account->content['sluser'] = array(
        '#type' => 'user_profile_category',
        '#title' => variable_get('sluser_profile_category_label',  'SlUser'),
      );
    }

    // reg key
    if (user_edit_access($account)) {
      $account->content['sluser']['regkey'] = array(
        '#type' => 'user_profile_item',
        '#title' => t('Reg Key'),
        '#value' => $sluser->reg_key,
        '#weight' => 1,
       );
    }

    // user name and links
    if ($sluser->id && $sluser->name != '' && $sluser->uuid != '') {
      $account->content['sluser']['name'] = array(
        '#type' => 'user_profile_item',
        '#title' => t('Name'),
        '#value' => $sluser->name,
        '#weight' => 0,
       );
       $account->content['sluser']['web_profile_link'] = array(
        '#type' => 'user_profile_item',
        '#title' => t('Web profile'),
        '#value' => '<a href="'. sluser_build_profile_link($sluser). '">'. t('Click here to see the web profile'). '</a>',
        '#weight' => 11,
       );
       $account->content['sluser']['inworld_profile_link'] = array(
        '#type' => 'user_profile_item',
        '#title' => t('Inworld profile'),
        '#value' => '<a href="secondlife:///app/agent/'. $sluser->uuid. '/about">'. t('Click here to see the inworld profile'). '</a>',
        '#weight' => 12,
       );
    }

  }
}

/**
* Build the web profile link
*/
function sluser_build_profile_link($sluser) {
  $link = variable_get('sluser_web_profile_link_model', 'http://world.secondlife.com/resident/%uuid');
  // define the path replacers
  $replacers = array('%uuid' =>$sluser->uuid, '%name' =>$sluser->name);
  // replace the strings
  foreach ($replacers as $key=>$value) {
    $link = str_replace($key, $value, $link);
  }
  return $link;
}

/**
* Edit user form
*/
function sluser_edit_user_form($account, $edit) {
  $form = array();

  // fieldset
  $form['sluser'] = array(
    '#type'          => 'fieldset',
    '#title'           => t('SlUser settings'),
    '#weight'       => 10,
    '#collapsible'  => TRUE,
  );

  // allow users to see the profile values
  $form['sluser']['sluser_allow_profile_view'] = array(
    '#type'               => 'checkbox',
    '#title'                => t('Allow profile view'),
    '#default_value'  => empty($edit['sluser_allow_profile_view']) ? $edit['sluser_allow_profile_view'] : variable_get('sluser_default_profile_view', 1),
    '#description'      => t('Allow other users to see your profile infos.'),
  );

  // allow users to see the sluser tab
  $form['sluser']['sluser_allow_tab_view'] = array(
    '#type'               => 'checkbox',
    '#title'                => t('Allow tab view'),
    '#default_value'  => empty($edit['sluser_allow_tab_view']) ? $edit['sluser_allow_tab_view'] : variable_get('sluser_allow_tab_view', 1),
    '#description'      => t('Allow other users to see your sluser tab.'),
  );

  // reg key renew
  $form['sluser']['renew_regkey_container'] = array(
    '#value' => '&nbsp;',
    '#prefix' => '<div id="renew-regkey-container">',
    '#suffix' => '</div>',
  );
  $form['sluser']['renew_regkey_button'] = array(
    '#type' => 'submit',
    '#value' => t('Renew Regkey'),
    '#description' => t("Push the button if you would like to renew your regkey."),
    '#weight' => 1,
    '#ahah' => array(
      'path' => 'sluser/users/renew-regkey/js',
      'wrapper' => 'renew-regkey-container',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );
  return $form;
}

/**
* Generates a regkey
*/
function sluser_generate_regkey($account) {
  // generate a reg key
      $reg_key = module_invoke('user', 'password');
      $values = array(
        'uid' => $account->uid,
        'reg_key' => $reg_key
      );
      //drupal_write_record('sluser', $values);
      sluser_save($values);
      drupal_set_message(t('SlUser updated : reg key is : '). $reg_key);
}

/**
* AHAH callback to renew the regkey
*/
function sluser_renew_regkey_js() {
  // drupal's alchemy
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form_state['post'] = $form['#post'] = $_POST;
  $form['#programmed'] = $form['#redirect'] = FALSE;
  $renew_regkey_form = $form['sluser']['renew_regkey_container'];
  unset($renew_regkey_form['#prefix'], $renew_regkey_form['#suffix']);
  // generate the regkey
  sluser_generate_regkey($form['_account']['#value']);
  // return the value
  $output = theme('status_messages') . drupal_render($renew_regkey_form);
  drupal_json(array('status' => TRUE, 'data' => $output));
}

/**
* Determine if a user can access to the contact tab.
*/
function _sluser_user_tab_access($account) {
  return user_edit_access($account) || $account->sluser_allow_tab_view;
}

/**
* Personal contact page.
*/
function sluser_user_page($account) {
  global $user;

  // check if the user has a sluser
  $sluser = sluser_load(array('uid' =>$account->uid));
  if (empty($sluser->id)) {
    drupal_set_message(t('User not registered.'));
    //return '';
  }

  // get the profile image
  // TODO : uncomment this when the new framework will be validated
  //$sluser->profile_image = secondlife_extract_picture_key(secondlife_get_linden_user_profile($sluser->key));

  // sluser base values
  //<div><img src="http://secondlife.com/app/image/'. $sluser->profile_image. '/1" alt="profile image"/></div>

  // build some usefull sentences
  $sluser->reg_key_description = t('Use this key to register inworld with a terminal. Select a terminal on !url', array('!url' => l(t('the terminals list page'), "sluser/terminals")));

  // define if the user can view some private infos
  $sluser->canedit = user_edit_access($account);

  // render the theme
  $output = theme('sluser_display', $sluser);

  // ask for others panels from others modules
  $panels = module_invoke_all('sluser_user_page', $account);
  foreach ($panels as $panel) {
    $output .= $panel;
  }
  return $output;
}

/**
* Preprocess the template for the display view
*/
function template_preprocess_sluser_display(&$variables) {
  $variables['sluser'] = $variables[0];
}

/**
* Load the data for a single sluser
*/
function sluser_load($array) {
  // Dynamically compose a SQL query:
  $query = array();
  $params = array();

  if (is_numeric($array)) {
    $array = array('uid' => $array);
  }
  elseif (!is_array($array)) {
    return FALSE;
  }

  foreach ($array as $key => $value) {
    if ($key == 'uid' || $key == 'id') {
      $query[] = "$key = %d";
      $params[] = $value;
    }
    else {
      $query[]= "LOWER($key) = LOWER('%s')";
      $params[] = $value;
    }
  }
  $query = db_query('SELECT * FROM {sluser} u WHERE '. implode(' AND ', $query), $params);

  if (!$sluser = db_fetch_object($query)) {
    $sluser = FALSE;
  }

  // get the linked user
  $account = user_load(array('uid' =>$sluser->uid));
  $account = drupal_unpack($account, 'data');
  $sluser->account = $account;

  return $sluser;
}

/**
* Store the sluser
*/
function sluser_save($values) {

  // convert values to an object
  $values = (object)$values;

  $response = array();
  // check if the sluser already exists
  $sluser = new stdclass;
  $data = array();
  if ($values->id != 0) {
    $data['id'] = $values->id;
  }
  if ($values->uid != 0) {
    $data['uid'] = $values->uid;
  }
  $sluser = sluser_load($data);

  if (!$sluser->id) {
    // insert new entry
    $response['op'] = drupal_write_record('sluser', $values);
  }
  else {
    // update entry
    $values->id = $sluser->id;
    $response['op'] = drupal_write_record('sluser', $values, 'id');
  }

  // define the result status
  $response['status'] = TRUE;
  switch ($response['op']) {
    case SAVED_NEW:
      $response['message'] = 'success;'. t('New user added.')."/".print_r($values,true);
      break;
    case SAVED_UPDATED:
      $response['message'] = 'success;'. t('User updated.');
      break;
    default:
      $response['status'] = FALSE;
      $response['message'] = 'error;'. t('Error saving user.').$response['op'];
      break;
  }
  $response['id'] = $values->id;
  return $response;
}

/**
* Delete the sluser.
*/
function sluser_delete($sluser_id) {
  // get the sluser
  $sluser = sluser_load(array('id' =>$sluser_id));

  // remove sluser
  db_query('DELETE FROM {sluser} WHERE id = %d', $sluser_id);

  return $sluser;
}